<?php

namespace Drupal\preprocessors\PluginManager\PreprocessorsPluginManager;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Plugin\DefaultPluginManager;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\preprocessors\PluginManager\PreprocessorsPluginManager\Annotation\Preprocessor as Annotation;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a Plugin Manager for Preprocessors.
 *
 * @package Drupal\preprocessors\PluginManager
 */
final class PreprocessorsPluginManager extends DefaultPluginManager implements ContainerInjectionInterface {

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) : PreprocessorsPluginManager {
    /* @noinspection PhpParamsInspection */
    return new static(
      $container->get('container.namespaces'),
      $container->get('cache.discovery'),
      $container->get('module_handler')
    );
  }

  /**
   * Constructs an PreprocessorsPluginManager object.
   *
   * @param \Traversable $namespaces
   *   An object that implements \Traversable which contains the root paths
   *   keyed by the corresponding namespace to look for plugin implementations.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache_backend
   *   Cache backend instance to use.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler to invoke the alter hook with.
   */
  public function __construct(
    \Traversable $namespaces,
    CacheBackendInterface $cache_backend,
    ModuleHandlerInterface $module_handler
  ) {
    parent::__construct(
      'Plugin/preprocessors',
      $namespaces,
      $module_handler,
      PreprocessorInterface::class,
      Annotation::class
    );

    $this->alterInfo('preprocessors_info');
    $this->setCacheBackend($cache_backend, 'preprocessors');
  }

  /**
   * Get Preprocessor definitions.
   *
   * {@inheritdoc}
   */
  public function getPreprocessors(): array {
    // Get definitions first.
    $definitions = $this->getDefinitions();

    // Variable to house formatted definitions.
    $preprocessors = [];

    // Format the definitions.
    foreach ($definitions as $definition) {
      // If the definition doesn't have a template set, we shouldn't return it.
      if (empty($definition[Preprocessor::TEMPLATE])) {
        continue;
      }

      // Set themes.
      $themes = $definition[Preprocessor::THEMES] ?? [Preprocessor::ALL_THEMES_VALUE];

      // Set the definition in the array according to all of its themes.
      foreach ($themes as $theme) {
        $preprocessors[$theme][$definition[Preprocessor::TEMPLATE]][$definition[Preprocessor::ID]] = $definition;

        // Sort the array of this theme.
        usort($preprocessors[$theme][$definition[Preprocessor::TEMPLATE]], 'self::sortPreprocessors');
      }
    }

    // Return the preprocessors.
    return $preprocessors;
  }

  /**
   * Sort preprocessors according to their weight.
   *
   * @noinspection
   *   PhpUnusedPrivateMethodInspection
   *
   * @param array $a
   *   First preprocessor definition.
   * @param array $b
   *   Second preprocessor definition.
   *
   * @return int
   *   Returns a value less than, equal to or greater than zero depending on if
   *   the first argument is considered to be respectively less than, equal to
   *   or greater than the second.
   */
  private function sortPreprocessors(array $a, array $b): int {
    return $a[Preprocessor::WEIGHT] ?? 0 - $b[Preprocessor::WEIGHT] ?? 0;
  }
}
